unit CheckBox98;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TCheckBoxState = (cbUnchecked, cbChecked, cbGrayed);

  TCheckBox98 = class(TGraphicControl)
  private
    { Private declarations }
    SignSpace: Integer;
    FAlignment: TLeftRight;
    FAllowGrayed: Boolean;
    FAutoSize: Boolean;
    FState: TCheckBoxState;
    function GetChecked: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetAllowGrayed(Value: Boolean);
    procedure SetAutoSize(Value: Boolean);
    procedure SetChecked(Value: Boolean);
    procedure SetState(Value: TCheckBoxState);
    procedure CMDialogChar (var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure WMLButtonDblClk(var Message: TMessage); message WM_LBUTTONDBLCLK;
  protected
    { Protected declarations }
    procedure AdjustBounds;
    procedure Click; override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Alignment: TLeftRight read FAlignment write SetAlignment default taRightJustify;
    property AllowGrayed: Boolean read FAllowGrayed write SetAllowGrayed default False;
    property AutoSize: Boolean read FAutoSize write SetAutoSize default True;
    property Caption;
    property Checked: Boolean read GetChecked write SetChecked default False;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State: TCheckBoxState read FState write SetState default cbUnchecked;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{$R CheckBox98.DCR}

type
  TLum = 0..240;

procedure Register;
begin
  RegisterComponents('Win98', [TCheckBox98]);
end;

constructor TCheckBox98.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque, csReplicatable];
  FAlignment := taRightJustify;
  FAutoSize := True;
  FState := cbUnchecked;
end;

destructor TCheckBox98.Destroy;
begin
  inherited;
end;

procedure TCheckBox98.CMDialogChar (var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled and Visible and
      ((KeyData and $20000000) <> 0) then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TCheckBox98.CMFontChanged(var Message: TMessage);
begin
  inherited;
  AdjustBounds;
end;

procedure TCheckBox98.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  AdjustBounds;
end;

procedure TCheckBox98.WMLButtonDblClk(var Message: TMessage);
begin
  inherited;
  Click;
end;

procedure TCheckBox98.AdjustBounds;
var
  Text: string;
var
  DC: HDC;
  Rect1, Rect2: TRect;
begin
  if not (csReading in ComponentState) and FAutoSize then
  begin
    Rect1 := ClientRect;
    DC := GetDC(0);
    Canvas.Handle := DC;
    Text := Caption;
    if ((Text = '') or (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
    Canvas.Font := Font;
    if not Enabled then
    begin
      OffsetRect(Rect1, 1, 1);
      DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect1,
        (DT_EXPANDTABS or DT_CALCRECT));
      OffsetRect(Rect1, -1, -1);
      DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect1,
        (DT_EXPANDTABS or DT_CALCRECT));
    end
    else
      DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect1,
        (DT_EXPANDTABS or DT_CALCRECT));
    ReleaseDC(0, DC);
  end;
  Rect2 := ClientRect;
  Canvas.Font := Font;
  Canvas.Font.Name := 'Wingdings';
  Canvas.Font.Size := (10 * Canvas.Font.Size) div 8;
  Text := '';
  DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect2,
    (DT_EXPANDTABS or DT_CALCRECT));
  if Rect2.Bottom > Rect1.Bottom then
    Rect1.Bottom := Rect2.Bottom;
  SignSpace := (Rect2.Right - Rect2.Left);
  if not (csReading in ComponentState) and FAutoSize then
  begin
    Rect1.Right := Rect1.Right + (4 * (Rect2.Right - Rect2.Left)) div 3;
    Canvas.Handle := 0;
    SetBounds(Left, Top, Rect1.Right, Rect1.Bottom);
  end;
end;

procedure TCheckBox98.Click;
begin
  inherited;
  case Ord(FState) of
    0: if FAllowGrayed then
         FState := cbGrayed
       else
         FState := cbChecked;
    1: FState := cbUnchecked;
    2: FState := cbChecked;
  end;
  Invalidate;
end;

procedure TCheckBox98.Paint;

  function Min(A, B: Integer): Integer;
  begin
    if A < B then
      Result := A
    else
      Result := B;
  end;

  function Max(A, B: Integer): Integer;
  begin
    if A > B then
      Result := A
    else
      Result := B;
  end;

  function LumColor(BasicColor: TColor; Level: TLum): TColor;
  var
    R, G, B, AA, BB: Integer;
    Lum: Extended;
  begin
    B := (BasicColor and $00FF0000) shr 16;
    G := (BasicColor and $0000FF00) shr 8;
    R := BasicColor and $000000FF;
    AA := Min(Min(R, G), B);
    BB := Max(Max(R, G), B);
    Lum := (AA * 240/255) + ((BB * 240/255) - (AA * 240/255)) / 2;
    if (R + G + B = 0) or (R + G + B = 3 * 255) then
    begin
      R := 127;
      B := 127;
      G := 127;
    end
    else
    if Lum <> 120 then
    begin
      if Lum > 120 then
      begin
        R := Round(255 - (255 - R) * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
        G := Round(255 - (255 - G) * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
        B := Round(255 - (255 - B) * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
      end
      else
      begin
        R := Round(R * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
        G := Round(G * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
        B := Round(B * 120 / (Min(Round(2 * Lum), Round(480 - 2 * Lum)) div 2));
      end;
    end;
    if Level <> 120 then
    begin
      if Level > 120 then
      begin
        R := Round(R + (255 - R) * (Level - 120) / 120);
        G := Round(G + (255 - G) * (Level - 120) / 120);
        B := Round(B + (255 - B) * (Level - 120) / 120);
      end
      else
      begin
        R := Round(R * Level / 120);
        G := Round(G * Level / 120);
        B := Round(B * Level / 120);
      end;
    end;
    Result := (Max(Min(B, 255), 0) shl 16) + (Max(Min(G, 255), 0) shl 8) +
      Max(Min(R, 255), 0);
  end;

const
  Alignments: array[0..1] of Word = (DT_LEFT, DT_RIGHT);
var
  Rect: TRect;
  Text: string;
  DrawStyle: Integer;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.FillRect(Classes.Rect(0, 0, Width, Height));
  Rect := ClientRect;
  Text := Caption;
  DrawStyle := DT_EXPANDTABS or DT_LEFT or DT_SINGLELINE or DT_VCENTER;
  if FAlignment = taRightJustify then
    Rect.Left := Rect.Left + (4 * SignSpace) div 3;
  Canvas.Brush.Style := bsClear;
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
  end
  else
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
  Rect := ClientRect;
  Canvas.Font.Name := 'Wingdings';
  Canvas.Font.Size := (10 * Canvas.Font.Size) div 8;
  Canvas.Font.Color := clBlack;
  Text := 'o';
  DrawStyle := DT_EXPANDTABS or Alignments[1 - Ord(FAlignment)] or DT_SINGLELINE or DT_VCENTER;
  DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
  Canvas.Brush.Style := bsSolid;
  if FState = cbGrayed then
    Canvas.Brush.Color := LumColor(clSilver, 210)
  else
    Canvas.Brush.Color := clWhite;
  if FAlignment = taRightJustify then
    Canvas.FloodFill(Rect.Left + SignSpace div 2,
      Rect.Top + (Rect.Bottom - Rect.Top) div 2, Canvas.Font.Color, fsBorder)
  else
    Canvas.FloodFill(Rect.Right - SignSpace div 2,
      Rect.Top + (Rect.Bottom - Rect.Top) div 2, Canvas.Font.Color, fsBorder);
  Canvas.Brush.Style := bsClear;
  if FState = cbGrayed then
    Canvas.Font.Color := clGray;
  if FState <> cbUnchecked then
  begin
    Text := '';
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
    Text := 'o';
    Canvas.Font.Color := clBlack;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, DrawStyle);
  end;
end;

function TCheckBox98.GetChecked: Boolean;
begin
  Result := (FState = cbChecked);
end;

procedure TCheckBox98.SetAlignment(Value: TLeftRight);
begin
  if Value <> FAlignment then
  begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TCheckBox98.SetAllowGrayed(Value: Boolean);
begin
  if Value <> FAllowGrayed then
  begin
    FAllowGrayed := Value;
    if FState = cbGrayed then
      FState := cbUnchecked;
    Invalidate;
  end;
end;

procedure TCheckBox98.SetAutoSize(Value: Boolean);
begin
  if Value <> FAutoSize then
  begin
    FAutoSize := Value;
    AdjustBounds;
  end;
end;

procedure TCheckBox98.SetChecked(Value: Boolean);
begin
  if (Value and (FState <> cbChecked)) or
    (not(Value) and (FState <> cbUnchecked)) then
  begin
    if Value then
      FState := cbChecked
    else
      FState := cbUnchecked;
    Invalidate;
  end;
end;

procedure TCheckBox98.SetState(Value: TCheckBoxState);
begin
  if (Value = cbGrayed) and not(FAllowGrayed) then
    Exit;
  if Value <> FState then
  begin
    FState := Value;
    Invalidate;
  end;
end;

end.
